<?php

namespace Pinpoint\RedirectManager\Controller\Adminhtml\Redirects;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Controller\ResultFactory;

/**
 * Class DownloadCsv
 * @package Pinpoint\RedirectManager\Controller\Adminhtml\Redirects
 */
class DownloadCsv extends Action
{
    /**
     * @var string
     */
    const EXPORT_FILENAME = 'redirects-export.csv';

    /**
     * @var \Pinpoint\RedirectManager\Model\ResourceModel\Redirect\Collection
     */
    protected $redirectCollection;

    /**
     * DownloadCsv constructor.
     * @param Context $context
     * @param \Pinpoint\RedirectManager\Model\ResourceModel\Redirect\Collection $redirectCollection
     */
    public function __construct(
        Context $context,
        \Pinpoint\RedirectManager\Model\ResourceModel\Redirect\Collection $redirectCollection
    ) {
        $this->redirectCollection = $redirectCollection;

        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Pinpoint_RedirectManager::redirect_manager');
    }

    /**
     * @return \Magento\Framework\Controller\Result\Raw
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Raw $result */
        $result = $this->resultFactory->create(ResultFactory::TYPE_RAW);
        $request = $this->getRequest();
        $collection = $this->redirectCollection;

        if (is_array($request->getParam('excluded')) && $excluded = $request->getParam('excluded')) {
            $collection->addFieldToFilter('id', ['nin' => array_values($excluded)]);
        }

        if (is_array($request->getParam('selected')) && $selected = $request->getParam('selected')) {
            $collection->addFieldToFilter('id', ['in' => array_values($selected)]);
        }

        if (is_array($request->getParam('filters'))) {
            $filters = $request->getParam('filters');
            unset($filters['placeholder']);
            foreach ($filters as $field => $filter) {
                $collection->addFieldToFilter($field, $filter);
            }
        }

        $result
            ->setHeader('Content-type', 'text/csv')
            ->setHeader('Content-Disposition', 'attachment; filename=' . self::EXPORT_FILENAME)
            ->setHeader('Pragma', 'no-cache')
            ->setHeader('Expires', '0')
            ->setHttpResponseCode(200)
        ;

        $output = "";
        $i = 0;

        foreach ($collection->getItems() as $redirect) {
            if (!$i) {
                $output .= implode(",", array_keys($redirect->getData())) . PHP_EOL;
            }
            $output .= implode(",", $redirect->getData()) . PHP_EOL;
            $i++;
        }

        $result->setContents($output);

        return $result;
    }
}
