<?php
namespace Pinpoint\RedirectManager\Controller\Adminhtml\Redirects;

use Magento\Backend\App\Action;
use Magento\Framework\Exception\NoSuchEntityException;
use Pinpoint\RedirectManager\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Save
 * @package Pinpoint\RedirectManager\Controller\Adminhtml\Redirects
 */
class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_RedirectManager::redirectAdd';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var \Pinpoint\RedirectManager\Model\RedirectRepository
     */
    protected $redirectRepository;

    /**
     * @var \Pinpoint\RedirectManager\Model\RedirectFactory
     */
    protected $redirectFactory;

    /**
     * Save constructor.
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     * @param \Pinpoint\RedirectManager\Model\RedirectRepository $redirectRepository
     * @param \Pinpoint\RedirectManager\Model\RedirectFactory $redirectFactory
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor,
        \Pinpoint\RedirectManager\Model\RedirectRepository $redirectRepository,
        \Pinpoint\RedirectManager\Model\RedirectFactory $redirectFactory
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->redirectRepository = $redirectRepository;
        $this->redirectFactory = $redirectFactory;

        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Pinpoint_RedirectManager::redirect_manager');
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();

        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['id'])) {
                $data['id'] = null;
            }

            /** @var \Pinpoint\RedirectManager\Model\Redirect $model */
            $model = $this->redirectFactory->create();

            $id = $this->getRequest()->getParam('id');
            if ($id) {
                try {
                    $model = $this->redirectRepository->getById($id);
                } catch (NoSuchEntityException $exception) {
                    unset($exception);
                }
            }

            $model->setData($data);

            try {
                $model->save();
                $this->messageManager->addSuccessMessage(__('The redirect has been saved.'));
                $this->dataPersistor->clear('pinpoint_redirectmanager_resourcemodel_redirect_collection');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/new', ['id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/');
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the data.'));
            }

            $this->dataPersistor->set('pinpoint_redirectmanager_resourcemodel_redirect_collection', $data);
            return $resultRedirect->setPath('*/*/new', ['id' => $this->getRequest()->getParam('id')]);
        }
        return $resultRedirect->setPath('*/*/');
    }    
}
